@@ -152,6 +152,11 @@ def apply(userFunc, inRats, outRats, otherargs=None, controls=None):
 
     numBlocks = int(numpy.ceil(float(rowCount) / controls.blockLen))
     
+    if controls.progress is not None:
+        controls.progress.setTotalSteps(100)
+        controls.progress.setProgress(0)
+    lastpercent = 0
+
     # Loop over all blocks in the RAT(s)
     for i in range(numBlocks):
         state.setBlock(i, controls.blockLen)
@@ -170,10 +175,48 @@ def apply(userFunc, inRats, outRats, otherargs=None, controls=None):
         # Clear block caches
         inBlocks.clearCache()
         outBlocks.clearCache()
+
+        if controls.progress is not None:
+            percent = int((i * 100) / numBlocks)
+            if percent != lastpercent:
+                controls.progress.setProgress(percent)
+                lastpercent = percent                
     
     outBlocks.finaliseRowCount(outputRatHandleNameList)
-    
+
+    if controls.progress is not None:
+        controls.progress.setProgress(100)
+
+def copyRAT(input, output, progress=None):
+    """
+    Given an input and output filenames copies the RAT from 
+    the input and writes it to the output.
+    """
+    from .rat import getColumnNames
+    inRats = RatAssociations()
+    outRats = RatAssociations()
         
+    inRats.inclass = RatHandle(input)
+    outRats.outclass = RatHandle(output)
+
+    controls = RatApplierControls()
+    controls.progress = progress
+
+    otherArgs = OtherArguments()
+    otherArgs.colNames = getColumnNames(input)
+    if len(otherArgs.colNames) > 0:
+        apply(internalCopyRAT, inRats, outRats, otherArgs, controls)
+
+def internalCopyRAT(info, inputs, outputs, otherArgs):
+    """
+    Called from copyRAT. Copies the RAT
+    """
+    for columnName in otherArgs.colNames:
+        data = getattr(inputs.inclass, columnName)
+        setattr(outputs.outclass, columnName, data)
+
+        usage = inputs.inclass.getUsage(columnName)
+        outputs.outclass.setUsage(columnName, usage)
 
 class RatHandle(object):
     """
@@ -251,6 +294,7 @@ class RatApplierControls(object):
         self.outRowCountMethod = RCM_EQUALS_INPUT
         self.fixedOutRowCount = None
         self.rowCountIncrementSize = None
+        self.progress = None
     
     def setBlockLength(self, blockLen):
         "Change the number of rows used per block"
@@ -297,6 +341,12 @@ class RatApplierControls(object):
         self.fixedOutRowCount = totalsize
         self.rowCountIncrementSize = incrementsize
 
+    def setProgress(self, progress):
+        """
+        Set the progress display object. Default is no progress
+        object. 
+        """
+        self.progress = progress
 
 class OtherArguments(object):
     """
